package exnihilocreatio.util;

import lombok.Getter;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.NonNullList;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.common.registry.ForgeRegistries;
import net.minecraftforge.oredict.OreDictionary;

import javax.annotation.Nonnull;

public class BlockInfo implements StackInfo {

    public static final BlockInfo EMPTY = new BlockInfo(ItemStack.field_190927_a);

    @Nonnull
    private IBlockState state;

    @Getter
    private boolean isWildcard = false;

    public BlockInfo(@Nonnull Block block) {
        this.state = getStateFromMeta(block, 0);
        checkWildcard();
    }

    public BlockInfo(@Nonnull Block block, int meta) {
        this.state = getStateFromMeta(block, meta);
        if (meta == -1 || meta == OreDictionary.WILDCARD_VALUE)
            this.isWildcard = true;
        else
            checkWildcard();
    }

    /**
     * Only use this if you need a very specific state
     * or want to compare against a very specific state,
     * otherwise use block, meta or just block
     */
    public BlockInfo(@Nonnull IBlockState state){
        this.state = state;
    }

    public BlockInfo(@Nonnull ItemStack stack) {
        this.state = getStateFromMeta(Block.func_149634_a(stack.func_77973_b()), stack.func_77952_i());
        checkWildcard();
    }

    public BlockInfo(@Nonnull String string) {
        if (string.isEmpty() || string.length() < 2) {
            this.state = Blocks.field_150350_a.func_176223_P();
            this.isWildcard = true;
            return;
        }
        String[] split = string.split(":");

        Block block;
        int meta = 0;

        switch (split.length) {
            case 1:
                block = Block.func_149684_b("minecraft:" + split[0]);
                break;
            case 2:
                try {
                    meta = split[1].equals("*") ? -1 : Integer.parseInt(split[1]);
                    block = Block.func_149684_b("minecraft:" + split[0]);
                } catch (NumberFormatException e) {
                    meta = 0;
                    block = Block.func_149684_b(split[0] + ":" + split[1]);
                } catch (NullPointerException e) {
                    this.state = Blocks.field_150350_a.func_176223_P();
                    this.isWildcard = true;
                    return;
                }
                break;
            case 3:
                try {
                    meta = split[2].equals("*") ? -1 : Integer.parseInt(split[2]);
                    block = Block.func_149684_b(split[0] + ":" + split[1]);
                } catch (NumberFormatException | NullPointerException e) {
                    this.state = Blocks.field_150350_a.func_176223_P();
                    this.isWildcard = true;
                    return;
                }
                break;
            default:
                this.state = Blocks.field_150350_a.func_176223_P();
                this.isWildcard = true;
                return;
        }

        if (block == null){
            this.state = Blocks.field_150350_a.func_176223_P();
            this.isWildcard = true;
        }
        else {
            if (meta == -1 || meta == OreDictionary.WILDCARD_VALUE) {
                this.state = getStateFromMeta(block, 0);
                this.isWildcard = true;
            }
            else {
                this.state = getStateFromMeta(block, meta);
                checkWildcard();
            }
        }
    }

    public static BlockInfo readFromNBT(NBTTagCompound tag) {
        Block item = ForgeRegistries.BLOCKS.getValue(new ResourceLocation(tag.func_74779_i("block")));
        int meta = tag.func_74762_e("meta");

        return item == null ? EMPTY : new BlockInfo(item, meta);
    }

    /**
     * This is a safe version of the block's getStateFromMeta
     * As it will attempt to get the meta state, but will
     * catch any errors and return a default state instead.
     */
    @Nonnull
    public static IBlockState getStateFromMeta(Block block, int meta){
        try {
            return block.func_176203_a(meta);
        } catch (Exception e) {
            return block.func_176223_P();
        }
    }

    private void checkWildcard(){
        // This checks if the block has sub items or not.
        // If not, accept any block that matches this, otherwise
        // Only accept blocks with meta 0
        NonNullList<ItemStack> subItems = NonNullList.func_191196_a();
        state.func_177230_c().func_149666_a(state.func_177230_c().func_149708_J(), subItems);
        if (subItems.size() <= 1)
            this.isWildcard = true;
    }

    //StackInfo

    @Override
    public String toString() {
        int meta = getMeta();
        return ForgeRegistries.BLOCKS.getKey(state.func_177230_c()) + (meta == 0 ? "" : (":" + meta));
    }

    @Nonnull
    @Override
    public ItemStack getItemStack() {
        Item item = Item.func_150898_a(state.func_177230_c());
        return item == Items.field_190931_a ? ItemStack.field_190927_a : new ItemStack(item, 1, getMeta());
    }

    @Override
    public boolean hasBlock() {
        return true;
    }

    @Nonnull
    @Override
    public Block getBlock() {
        return state.func_177230_c();
    }

    @Nonnull
    @Override
    public IBlockState getBlockState() {
        return state;
    }

    @Override
    public int getMeta() {
        return isWildcard ? -1 : state.func_177230_c().func_176201_c(state);
    }

    @Override
    public NBTTagCompound writeToNBT(NBTTagCompound tag) {
        tag.func_74778_a("block", ForgeRegistries.BLOCKS.getKey(state.func_177230_c()).toString());
        tag.func_74768_a("meta", state.func_177230_c().func_176201_c(state));

        return tag;
    }

    @Override
    public boolean isValid() {
        return this.state != Blocks.field_150350_a.func_176223_P();
    }

    @Override
    public int hashCode() {
        return state.hashCode();
    }

    /**
     * Equals will vary base on Wildcard value. If wildcard is true
     * this will only compare blocks, otherwise it will also factor
     * in meta values.
     */
    @Override
    public boolean equals(Object other) {
        if (isWildcard){
            if (other instanceof BlockInfo)
                return Block.func_149680_a(state.func_177230_c(), ((BlockInfo) other).getBlock());
            else if (other instanceof ItemInfo)
                return Block.func_149680_a(state.func_177230_c(), ((ItemInfo) other).getBlock());
            else if (other instanceof ItemStack)
                return Block.func_149680_a(state.func_177230_c(), Block.func_149634_a(((ItemStack) other).func_77973_b()));
            else if (other instanceof Block)
                return Block.func_149680_a(state.func_177230_c(), (Block)other);
            else if (other instanceof ItemBlock) {
                return Block.func_149680_a(state.func_177230_c(), ((ItemBlock) other).func_179223_d());
            }
        }
        else {
            if (other instanceof BlockInfo)
                return state == ((BlockInfo) other).state;
            else if (other instanceof ItemInfo)
                return state == ((ItemInfo) other).getBlockState();
            else if (other instanceof ItemStack)
                return state == BlockInfo.getStateFromMeta(Block.func_149634_a(((ItemStack) other).func_77973_b()), ((ItemStack) other).func_77952_i());
            else if (other instanceof Block)
                return Block.func_149680_a(state.func_177230_c(), (Block)other);
            else if (other instanceof ItemBlock) {
                return Block.func_149680_a(state.func_177230_c(), ((ItemBlock) other).func_179223_d());
            }
        }

        return false;
    }
}
